@make(ais)
@aiInfo(topic=<Pragma INTERFACE: allowed names and illegalities>,
number=[AI-00306/15],
classcode=[BI],
statuscode=[WJ],
mainref=[13.09@\(03)],
date=[88-05-23])
@aiHeading()
@begin(history)
@begin(toplines)
@mainref()
@standard[02.08@\(09)]
!class binding interpretation 84-10-16
!status approved by WG9/AJPO 88-02-05
@end(toplines)
!status approved by Director, AJPO 88-02-05
!status approved by WG9/Ada Board 87-12-07
!status approved by Ada Board 87-07-30
!status panel/committee-approved 87-05-06 (reviewed)
!status panel/committee-approved (10-0-3) 87-02-17 (by ballot) (pending editorial review)
!status panel/committee-approved (7-0-2) 86-11-13 (pending letter ballot)
!status work-item 86-09-10
!status failed letter ballot (2-9-2) 86-09
!status committee-approved (8-0-1) 86-05-13 (pending letter ballot)
!status committee-approved (6-1-0) 86-02-21 (pending editorial review)
!status work-item 86-01-24
!status received 84-10-16
@references[!references @CommentNum[83-00445]83-00445, @CommentNum[83-00492]83-00492, @CommentNum[83-00637]83-00637, @CommentNum[83-00697]83-00697, @CommentNum[83-00718]83-00718, @CommentNum[83-00726]83-00726, @CommentNum[83-00790]83-00790]
@topic{!topic @value(aitopic)}
@end(history)

@subheading[!summary 87-08-20]

If a pragma INTERFACE names a language that is acceptable to an
implementation, the subprogram name must denote one or more subprograms
declared explicitly earlier in the same declarative part or package
specification.  (The pragma has no effect if no named subprogram satisfies the
requirements.)  The pragma is applied to all such subprograms other than
enumeration literals and subprograms declared by generic instantiation.

If a subprogram named in the pragma was declared by a renaming declaration,
the pragma  applies  to the  denoted  subprogram,  but only  if  the  denoted
subprogram otherwise satisfies  the above requirements.

It is illegal to apply a pragma INTERFACE to a subprogram for which a pragma
INTERFACE has already been applied.

If a pragma INTERFACE applies to a subprogram, it is illegal to provide a
body for the subprogram.

@subheading[!question 86-07-01]

The pragma INTERFACE is applied to a subprogram. Can the pragma be applied to
any of the following subprograms?
@begin(enum-letter0)
enumeration literals

attributes that denote functions

predefined operators

derived subprograms

subprogram names declared by renaming declarations

subprograms declared by generic instantiations

subprograms declared by subprogram_@|body declarations
@end(enum-letter0)
If an overloaded name given in a pragma INTERFACE denotes several subprograms
of which only a few satisfy the requirements for the pragma, to which
subprograms does the pragma apply, if any?

Is a pragma INTERFACE ignored if it names a subprogram that has a body, or is
it illegal to provide such a body?

What is the effect if a pragma INTERFACE is given more than once for the same
subprogram?

@subheading[!recommendation 87-08-20]

If a pragma INTERFACE names a language that is acceptable to an
implementation, the subprogram name must denote one or more subprograms
declared explicitly earlier in the same declarative part or package
specification.  The pragma is applied to all such subprograms other than
enumeration literals and subprograms declared by generic instantiation.

It is illegal to apply a pragma INTERFACE to a subprogram for which a pragma
INTERFACE has already been applied.

If a pragma INTERFACE applies to a subprogram, it is illegal to provide a
body for the subprogram.

@subheading[!discussion 86-08-20]

The pragma INTERFACE gives a means of providing a subprogram body other than
by a subprogram body declaration.  The pragma was only intended to be applied
to subprograms for which users can provide bodies.  In particular, since an
explicit subprogram body declaration cannot be provided for a subprogram that
is an enumeration literal, an attribute, a predefined operator, or a derived
subprogram, it was not intended that the pragma apply to such subprograms.
For example:
@begin(example)
@key[declare]
     @key[type] INT @key[is] @key[range] 1..10;
     @key[pragma] INTERFACE (FORTRAN, "+");   -- ignored
@end(example)
The pragma is ignored since the only subprogram @dq{+} declared earlier in this
declarative part is the implicitly declared predefined @dq{+}.

13.9(3) says that the subprogram name in the pragma INTERFACE is allowed to
@dq{stand for} several overloaded subprograms.  Suppose the subprogram name
stands for several subprograms, not all of which are declared earlier in the
same declarative part or package specification:
@begin(example)
@key[procedure] P (B : BOOLEAN);       -- P.1
@key[package] R @key[is]
     @key[procedure] P (I : INTEGER);  -- P.2
     @key[pragma] INTERFACE (XXX, P);
@end(example)
The intent is that the pragma INTERFACE be applied only to P.2 since P.2 is
the only subprogram that is declared earlier in the same package
specification; a body must be provided for P.1.

In addition, if a subprogram is overloaded, the pragma INTERFACE applies only
to those subprograms for which bodies can be provided, e.g.:
@begin(example)
@key[package] P1 @key[is]
     @key[type] ENUM @key[is] (A, B, C);
     @key[function] B @key[return] INTEGER;
     @key[pragma] INTERFACE (XXX, B);
@key[end] P1;
@end(example)
The pragma only applies to function B if it is supported for language XXX; it
does not apply to enumeration literal B.  Similarly:
@begin(example)
@key[package] T @key[is]
    @key[function] P @key[return] INTEGER;
@key[end] T;
@hinge()

@key[package] @key[body] T @key[is]
    @key[procedure] P;
    @key[function] P @key[return] INTEGER @key[is]
    @key[begin] ... @key[end] P;
    @key[pragma] INTERFACE (XXX, P);
@key[begin]
    ...
@end(example)
In this case, the pragma only applies to procedure P since function P is not
declared earlier in the same declarative part.  (Function P is declared in
the package specification.)

13.9(3) also says:
@begin(quotation)
A body is not  allowed for such a  subprogram (not even in  the
form of a body stub)  since the instructions of the  subprogram
are written in another language.
@end(quotation)
This restriction means that if the pragma is accepted and is applied to
certain subprograms, it is illegal to provide a body for any of these
subprograms.  For example:
@begin(example)
@key[package] P2 @key[is]
     @key[procedure] R (B : BOOLEAN);         -- R.1
     @key[procedure] R (I : INTEGER);         -- R.2
     @key[pragma] INTERFACE (YYY, R);         -- (1)
@key[end] P2;
@hinge()

@key[package] @key[body] P2 @key[is]
     @key[procedure] R (B : BOOLEAN) @key[is] ...;  -- (2); illegal
@key[end] P2;
@end(example)
Since the pragma at (1) specifies that a body for R.1 and R.2 is supplied in
language YYY, it is illegal to supply a body for either R.1 (as in (2)) or
R.2.  Similar illegalities can arise in a declarative part:
@begin(example)
@key[declare]
     @key[procedure] R (B : BOOLEAN);         -- R.1
     @key[procedure] R (I : INTEGER);         -- R.2
     @key[pragma] INTERFACE (YYY, R);
@hinge()

     @key[procedure] R (B : BOOLEAN) @key[is] ...;  -- illegal
@end(example)
It is immaterial whether the pragma appears before or after the body:
@begin(example)
@key[declare]
     @key[procedure] R (B : BOOLEAN);         -- R.1
     @key[procedure] R (I : INTEGER);         -- R.2
     @key[procedure] R (B : BOOLEAN) @key[is] ...;  -- illegal
     @key[pragma] INTERFACE (YYY, R);
@key[begin]
@end(example)
If the pragma is accepted for language YYY, the pragma applies to R.1 and R.2
so it is illegal to provide a body for R.1 (or R.2).  It is similarly illegal
to provide a body for R.1 even if the original declaration of R.1's
specification is deleted:
@begin(example)
@key[declare]
     @key[procedure] R (I : INTEGER);         -- R.2
     @key[procedure] R (B : BOOLEAN) @key[is] ...;  -- illegal body for R.1
     @key[pragma] INTERFACE (YYY, R);
@key[begin]
@end(example)
The pragma applies to subprogram R.1 as well as to R.2, since R.1 is declared
explicitly (by the subprogram_@|body declaration).  Since the subprogram body
declaration also provides a body for R.1, and since it is intended to be
illegal to provide a body for a subprogram to which the pragma applies, the
declarative part is illegal.

Suppose the pragma INTERFACE is given more than once for the same subprogram:
@begin(example)
@key[package] P2 @key[is]
     @key[procedure] P;
     @key[pragma] INTERFACE (L1, P);
     @key[function] P @key[return] INTEGER;
     @key[pragma] INTERFACE (L1, P);     -- illegal
@key[end] P2;
@end(example)
If L1 is an acceptable language, the second pragma would apply to procedure P
as well as to function P.  Since the pragma in effect provides a body for the
subprograms to which it applies, and since two bodies cannot be given for
the same subprogram, the second pragma is illegal.

If the subprogram named in the pragma was declared by a renaming declaration,
the pragma applies to the denoted subprogram, but only if the denoted
subprogram otherwise satisfies the requirements, i.e., the denoted subprogram
must be explicitly declared earlier in the same declarative part or package
specification and must not be an enumeration literal or a generic instance.
The ability to use names declared by renaming declarations makes it easier to
supply an Ada body for only one of two overloaded subprograms:
@begin(example)
@key[package] P3 @key[is]
     @key[function] OVERLOADED @key[return] INTEGER;
     @key[procedure] OVERLOADED;
     @key[procedure] NO_@|BODY @key[renames] OVERLOADED;
     @key[pragma] INTERFACE (LANGUAGE, NO_@|BODY);
     ...
@key[end] P3;
@end(example)
The pragma applies just to the subprogram denoted by NO_@|BODY, i.e., the
procedure.  A body must still be provided for the function OVERLOADED.

